/**
 * The ConvexTest applet is a graphic test of R2Convex class.
 *
 * <applet code="ConvexTest" width=420 height=455>
 * </applet>
 *
 * The applet nees JDK 1.1.
 * Tested in RedHat Linux 4.2 (JDK 1.1.3 and 1.1.5) and
 * under Win95, JDK 1.1.5.
 *
 */

//
//-----------------------------------------------------------
//This is the fourth programm for the examination
//It is made by VILenin (with the help of Arty)
//-----------------------------------------------------------
//

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.util.Enumeration;
import R2Graphics.*;
import R2Convex;

public class ConvexTest extends Applet {
    R2Convex convex;
    Label areaField;
    Label perimeterField;

//
//--------------------------------------------------
    Label testPointField;
//--------------------------------------------------
//

    Button addPointButton;
    Button initButton;

    TextField inputPointX;
    TextField inputPointY;

    Rectangle planeRect;
    Canvas plane;

    R2Transformation mapping; // Real coordinates --> pixel coordinates
    R2Transformation inverseMapping;

//
//-----------------------------------------------------------
    R2Point testPoint;        //New point for test (in or out)
//-----------------------------------------------------------
//

    // Layout constants
    static final int leftMargin   = 5;
    static final int topMargin    = 5;
    static final int vertSkip     = 5;
    static final int horSkip      = 5;
    static final int labelWidth   = 200;
    static final int labelHeight  = 30;
    static final int buttonWidth  = 100;
    static final int numberWidth  = 60;
    static final int numberLabelWidth = 20;
    static final int planeWidth   = 2 * labelWidth + horSkip;
    static final int planeHeight  = (planeWidth * 8) / 10;

    // Coordinate mapping constants
    static final double XMIN = -10.;
    static final double XMAX = 10.;
    static final double YMIN = XMIN * 0.8;
    static final double YMAX = XMAX * 0.8;

    public static void main(String[] args) {
        Frame f = new Frame("Convex");

        WindowAdapter wa = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        };
        f.addWindowListener(wa);

        f.setFont(new Font("Helvetica", Font.PLAIN, 14));

        // Add menu bar
        final MenuBar mb = new MenuBar();
        f.setMenuBar(mb);
        final Menu fileMenu = new Menu("File");
        final MenuItem quitItem = new MenuItem("Quit");
        fileMenu.add(quitItem);
        mb.add(fileMenu);

        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // System.out.println("Action " + e);
                if (e.getActionCommand().equals("Quit")) {
                    System.exit(0);
                }
            }
        };
        fileMenu.addActionListener(al);

        ConvexTest ct = new ConvexTest();
        f.add("Center", ct);

        // f.pack();
        f.setSize(
            2*leftMargin + planeWidth + 10,
            planeHeight + 3*labelHeight + 4*vertSkip + 2*topMargin +
            50      // For menu bar
        );
        f.setLocation(20, 60);

        ct.init();
        ct.start();

        f.setVisible(true);
    }

    public ConvexTest() {
        convex = new R2Convex();
    }

    public void init() {
        setLayout(null);

        setFont(new Font("Helvetica", Font.PLAIN, 14));
        setBackground(Color.lightGray);

        int x = leftMargin;
        int y = topMargin;

        areaField = new Label("Area: ");
        add(areaField);
        areaField.setBounds(x, y, labelWidth, labelHeight);
        x += labelWidth + horSkip;

        perimeterField = new Label("Perimeter: ");
        add(perimeterField);
        perimeterField.setBounds(x, y, labelWidth, labelHeight);
        x = leftMargin;
        y += labelHeight + vertSkip;

//
//---------------------------------------------------------
	testPointField = new Label("");
	add(testPointField);
	testPointField.setBounds(x, y, 2*labelWidth + horSkip, labelHeight);
	y += labelHeight + vertSkip;
//---------------------------------------------------------
//

        addPointButton = new Button("Add a point");
        add(addPointButton);
        addPointButton.setBounds(x, y, buttonWidth, labelHeight);
        x += buttonWidth + 2*horSkip;

        Label coord = new Label("X:");
        add(coord);
        coord.setBounds(x, y, numberLabelWidth, labelHeight);
        x += numberLabelWidth + horSkip;

        inputPointX = new TextField();
        add(inputPointX);
        inputPointX.setBounds(x, y, numberWidth, labelHeight);
        x += numberWidth + 2*horSkip;

        coord = new Label("Y:");
        add(coord);
        coord.setBounds(x, y, numberLabelWidth, labelHeight);
        x += numberLabelWidth + horSkip;

        inputPointY = new TextField();
        add(inputPointY);
        inputPointY.setBounds(x, y, numberWidth, labelHeight);
        x += numberWidth + 2*horSkip;

        initButton = new Button("Init");
        add(initButton);
        int w = planeWidth + horSkip - x;
        initButton.setBounds(x, y, w, labelHeight);

        x = leftMargin;
        y += labelHeight + vertSkip;

        planeRect = new Rectangle(x, y, planeWidth, planeHeight);
        plane = new MyPlane();
        add(plane);
        plane.setBounds(planeRect);
        plane.setBackground(Color.white);

//
//-------------------------------------------------------------
      	testPoint = new R2Point (0., 0.);
//-------------------------------------------------------------
//
			
  
        MyMouseAdapter mouseAdapter = new MyMouseAdapter();
        plane.addMouseListener(mouseAdapter);

        MyActionListener al = new MyActionListener();

        // Add action listener for "Add a point" button
        addPointButton.addActionListener(al);
        initButton.addActionListener(al);
        inputPointX.addActionListener(al);
        inputPointY.addActionListener(al);

        //
        // Initialize the coordinate mapping
        //
        double xcoeff = ((double) planeWidth) / (XMAX - XMIN);
        double ycoeff = ( -((double) planeHeight) / (YMAX - YMIN) );
        mapping = new R2Transformation( // Real coord. --> pixel coord.
            new R2Matrix(
                xcoeff, 0.0,
                0.0, ycoeff
            ),
            new R2Vect(
                ((double) planeWidth) / 2.,
                ((double) planeHeight) / 2.
            )
        );

        inverseMapping = mapping.inverse();   // pixel coord. --> real coord.
    }

    class MyPlane extends Canvas {

        public void paint(Graphics g) {
            // Draw cells
            double x = 0.;
            while (x < XMAX) {
                if (x == 0.)
                    g.setColor(Color.black); // Coord. axis
                else
                    g.setColor(Color.lightGray);

                R2Point p1 = new R2Point(x, YMIN);
                R2Point p2 = new R2Point(x, YMAX);
                R2Point q1 = mapping.map(p1);
                R2Point q2 = mapping.map(p2);
                g.drawLine(
                    (int)q1.x, (int)q1.y,
                    (int)q2.x, (int)q2.y
                );

                if (x != 0.) {
                    p1.x = (-x);
                    p2.x = (-x);
                    q1 = mapping.map(p1);
                    q2 = mapping.map(p2);
                    g.drawLine(
                        (int)q1.x, (int)q1.y,
                        (int)q2.x, (int)q2.y
                    );
                }

                x += 1.;
            }

            double y = 1.;
            while (y < YMAX) {
                if (y == 0.)
                    g.setColor(Color.black); // Coord. axis
                else
                    g.setColor(Color.lightGray);

                R2Point p1 = new R2Point(XMIN, y);
                R2Point p2 = new R2Point(XMAX, y);
                R2Point q1 = mapping.map(p1);
                R2Point q2 = mapping.map(p2);
                g.drawLine(
                    (int)q1.x, (int)q1.y,
                    (int)q2.x, (int)q2.y
                );

                if (y != 0.) {
                    p1.y = (-y);
                    p2.y = (-y);
                    q1 = mapping.map(p1);
                    q2 = mapping.map(p2);
                    g.drawLine(
                        (int)q1.x, (int)q1.y,
                        (int)q2.x, (int)q2.y
                    );
                }

                y += 1.;
            }

            // Draw coordinate axes
            g.setColor(Color.black);
            R2Point p1 = new R2Point(0., YMIN);
            R2Point p2 = new R2Point(0., YMAX);
            R2Point q1 = mapping.map(p1);
            R2Point q2 = mapping.map(p2);
            g.drawLine(
                (int)q1.x, (int)q1.y,
                (int)q2.x, (int)q2.y
            );

            p1.x = XMIN; p1.y = 0.;
            p2.x = XMAX; p2.y = 0.;
            q1 = mapping.map(p1);
            q2 = mapping.map(p2);
            g.drawLine(
                (int)q1.x, (int)q1.y,
                (int)q2.x, (int)q2.y
            );

            drawConvex(g);

//
//-------------------------------------------------------------
  	    drawTestPoint (g, testPoint);
//-------------------------------------------------------------
//
        }

//
//-------------------------------------------------------------
//Draws our testpoint on the screen
//			
	private void drawTestPoint (Graphics g, R2Point p) {
   	    g.setColor (Color.blue);
	    g.drawOval ((int) p.x, (int) p.y, 3, 3);
	}
//-------------------------------------------------------------
//

        private void drawConvex(Graphics g) {
            int x0 = planeRect.x;
            int y0 = planeRect.y;
            g.setColor(Color.red);

            Enumeration vertices = convex.elements();

            R2Point q0 = new R2Point();
            R2Point qPrev = new R2Point();

            // The first vertex
            if (vertices.hasMoreElements()) {
                R2Point p0 = (R2Point) vertices.nextElement();

                // Draw the first point
                q0 = mapping.map(p0);
                qPrev = q0;
                g.drawLine(
                    (int)q0.x, (int)q0.y,
                    (int)q0.x, (int)q0.y
                );
            } else {
                return; // Convex is empty
            }

            while (vertices.hasMoreElements()) {
                R2Point p = (R2Point) vertices.nextElement();

                // Draw the edge
                R2Point q = mapping.map(p);
                g.drawLine(
                    (int)qPrev.x, (int)qPrev.y,
                    (int)q.x, (int)q.y
                );
                qPrev = q;
            }

            // Close the convex
            g.drawLine((int)qPrev.x, (int)qPrev.y, (int)q0.x, (int)q0.y);
            g.drawLine(
                (int)qPrev.x, (int)qPrev.y,
                (int)q0.x, (int)q0.y
            );
        }
    }

    public void start() {

    }

    public void run() {

    }

    // Action listeners
    class MyActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();
            if (source == addPointButton) {
                onAddPoint();
            } else if (source == initButton) {
                onInit();
            } else if (source == inputPointX || source == inputPointY) {
                onAddPoint();
            }
        }
    }

    // Mouse listener
    class MyMouseAdapter extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
                // Left mouse button is pressed
                Point q = e.getPoint();
                R2Point p = inverseMapping.map(new R2Point(q));
                addPoint(p);

//
//----------------------------------------------------
		findAngle();
            } else if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) {
		//Right mouse button is pressed
		testPoint = new R2Point(e.getPoint());
		plane.repaint();
		findAngle();
	    }
//----------------------------------------------------
//						

        }
    }

    void onAddPoint() {
        String textX = inputPointX.getText();
        String textY = inputPointY.getText();

        try {

            double x = Double.valueOf(textX).doubleValue();
            double y = Double.valueOf(textY).doubleValue();
            addPoint(new R2Point(x, y));

//
//-------------------------------------------------------
	    findAngle();
//-------------------------------------------------------
//

        } catch (NumberFormatException e) {
            // System.out.println("Input number: Illegal format: " + e.toString());
            return;
        }
    }

    void addPoint(R2Point t) {
        convex.add(t);
        areaField.setText("Area: " + (float) convex.area());
        perimeterField.setText("Perimeter: " + (float) convex.perimeter());

        inputPointX.setText("");
        inputPointY.setText("");

        plane.repaint();
        inputPointX.requestFocus();
    }

    private void onInit() {
        convex.init();

        areaField.setText("Area:");
        perimeterField.setText("Perimeter:");

        inputPointX.setText("");
        inputPointY.setText("");

//
//----------------------------------------------------------
	testPoint = new R2Point(0., 0.);
	testPointField.setText("");
//----------------------------------------------------------
//

        plane.repaint();
    }

//
//----------------------------------------------------------
    private void findAngle() {	
	testPointField.setText("The angle equals " + convex.angle(inverseMapping.map(testPoint)) + " degrees.");
    }
//----------------------------------------------------------
//

}
